#!python3
"""
=====
Lukis
=====

Description:
~~~~~~~~~~~~
* Generate device independent vector graphic stream
  that can be converted to pdf, svg, or other vector graphic format.

* The output stream is in pure txt, and csv format.

* A simple stream looks like this::

    MoveTo,50,50
    LineTo,150,150
    Stroke

* "Lukis" is the Malay word for "Draw"

* show(GStream) generates convert the graphic stream to pdf or svg or other
  graphic file, and open it using the default viewer.

Example:
--------
::

    #To make a line from (0,0) to (300,300) and display the graphics:
    S = moveto(0,0)\\
    + lineto(300,300)\\
    + stroke()
    show(S)


Installation
~~~~~~~~~~~~
Python 3 as default Python::

	pip install lukis

Python as Python3::

	pip3 install kopi


Coding habits:
--------------
* For fixed/static length array inputs like (width, height), (origin_x, origin_y) etc,
  uses tuple (__, __)
* For dynamic length array like [x0, x1, x2, ...xn],
  use list or numpy array [__, __, ...]

"""
__all__ = ["lineto", "moveto", "curveto", "stroke", "show",
            "line","setlinewidth", "setlinecolor",
            "setfillcolor",
            "move", "rotate",
            "setfont", "text", "ftext",
            "rectangle"]
__version__ = "0.18.5"

from .lukis import *



#!python3
"""
    =======
    symbols
    =======
    A library of static symbols that can be imported to your work.
    The symbols are defined in valid stream text format, so it should
    be easily extended even without the assistance of other program
"""
"""
    Reference
    =========

    # pdf generic commands
    LineTo = "LineTo"
    MoveTo = "MoveTo"
    Stroke = "Stroke"
    Close = "Close"
    Close_Stroke = "Close_Stroke"
    Fill = "Fill"
    Fill_Stroke = "Fill_Stroke"
    Close_Fill_Stroke = "Close_Fill_Stroke"

    Translate = "Translate"
    CurveTo = "CurveTo"
    RotateCoordinate = "RotateCoordinate"
    SetLineWidth = "SetLineWidth"
    SetDash = "SetDash"
    SetLineJoin = "SetLineJoin"
    SetLineColor = "SetLineColor"
    SetFillColor = "SetFillColor"
    SetFont = "SetFont"
    Text= "Text"
    Picture = "Picture"


    Important
    ~~~~~~~~~
    The end of the final operation must be terminated with EOL \n
"""
Arrow_Down = "MoveTo,0,0\n"\
           + "LineTo,10,30\n"\
           + "LineTo,-10,30\n"\
           + "Close\n"\
           + "Fill\n"

Arrow_Up = "MoveTo,0,0\n"\
         + "LineTo,10,-30\n"\
         + "LineTo,-10,-30\n"\
         + "Close\n"\
         + "Fill\n"



__all__=["logpretty", "mpretty"]
import math
import numpy as np
def logpretty(x = [],VerbalEcho=False):
    xmin = 10.0** np.floor(np.log10( min(x) ) )
    xmax = 10.0** np.ceil(np.log10( max(x)))
    NumberOfChoice = int(np.log10(xmax/xmin)) + 1
    Output = np.zeros(NumberOfChoice)
    for n in range(NumberOfChoice):
        Output[n] = xmin*10**n
    return Output

def pretty(x=[], n=6, font_pt=10, xWidth = 100, VerbalEcho=False):
    """
    To generate a set a pretty value for assigning x or y ticks
    A Near equivalent to R pretty function.
    But I allow the xmax to be not always covered by the last tick,
    so the plotted area is actually larger and, in my taste, prettier.
    There is a number of existing non-trivial algorithm for this.
    Should take a read a have some comparison in the future.
    So far, may not be able to handle Log axis.
    Use [1, 2, 5] as the pretty base candidate for Spacing
    n is the number of desired tick number
    preliminary one. xmin->hard fixed, xmax-> flexibly tuned
    """
    n = round(n)
    xmin = min(x)
    xmax = max(x)
    deltax=xmax-xmin
    MindlessSpacing = deltax/n
    order = ( round(math.log10(MindlessSpacing)) )
    #PrettyCandidates = np.array([0.1, 0.2, 0.25, 0.5, 1, 2, 2.5, 5]) * (10**order)
    PrettyCandidates = np.array([0.1, 0.2, 0.5, 1, 2, 5]) * (10**order)
    NumOfChoice = len(PrettyCandidates)
    PossibleN = np.ceil( deltax/PrettyCandidates )
    OverNumber = abs(n-PossibleN)
    Overshoot = PossibleN * PrettyCandidates - deltax
    FontDigits =  abs(order) + 1
    weight_OverNumber =  font_pt*FontDigits
    weight_Overshoot = 2* xWidth/deltax
    OvershootPenalty= Overshoot*weight_Overshoot
    OverNumberPenalty = OverNumber * weight_OverNumber
    TotalPenalty = OverNumberPenalty + OvershootPenalty
    CandidateID = np.argmin(TotalPenalty)     #what if more than one ?
                        #Dont worry, .index will take care of it.
                        #It only returns One value :)
    PrettySpacing =  PrettyCandidates[CandidateID]
    N=int(PossibleN[CandidateID])

    # My taste is that: the xmin must be covered by the first tick
    PrettyStarting = math.floor( xmin/PrettySpacing ) * PrettySpacing
    PrettyEnding = PrettyStarting + N*PrettySpacing
    #After flooring for the pretty starting value, the Last tick will be left shifted,
    # and in the worse case, sway much away from the xmax.
    #Just in case, lets see if it is neccesary to compensate the End Value
    if (xmax - PrettyEnding) > (PrettySpacing/2) :
        N += 1
        PrettyEnding = PrettyStarting + N*PrettySpacing
    Output= np.zeros(N+1)
    for i in range(N+1):
        Output[i] = PrettyStarting + i*PrettySpacing
    if VerbalEcho == True:
        print (x)
        print ("n : " , n)
        print ("Mindless Spacing d : " , MindlessSpacing)
        print (("Candidates : " , PrettyCandidates))
        print (("Possible N : "  , PossibleN))
        print (("Font Digits : " ,  FontDigits))
        print (("OverNumber : " , OverNumber))
        print ("OverNumberPenalty : " , OverNumberPenalty)
        print (("Overshoot : " , Overshoot))
        print ("Overshoot Penalty : " , OvershootPenalty)
        print (("Penalty : " , TotalPenalty))
        print ("PrettySpacing : ", PrettySpacing)
        print ("N : " , N)
        print ("PrettyStarting : " , PrettyStarting)
        print ("PrettyEnding : " , PrettyEnding)
        print ("xmax-PrettyEnding : " , xmax - PrettyEnding)
        print ("Pretty Output : " , Output)
    return Output



#python3
def FindBlock(S, Opening, Ending, StartingPoint = 0):
    """
    Find the first start index and the end index of the bound by
    Opening and Ending strings

    Example:
    S = "aa11xxxx11"
    FindBlock(S, "11", "11")
    returns (2, 8) .

    Returns (-1,-1) if not found or error
    """
    StartIndex = S.find(Opening, StartingPoint)
    if StartIndex == -1:
        return (-1, -1)

    EndIndex = S.find(Ending, StartIndex + 1)
    if EndIndex == -1:
        return (-1,-1)
    return (StartIndex, EndIndex)
def GetTheLineBeforeCommentBlocks(S, CommentStart, CommentEnd):
    Result = []
    (a, b) = FindBlock(S, CommentStart, CommentEnd, 0)
    while(a != -1):
        #print("-"*10 +  " Block "+ str(BlockIndex) +  "-"*20)
        end_of_previou_line  = S.rfind("\n", 0, a)
        start_of_previou_line = S.rfind("\n", 0, end_of_previou_line)
        Result +=  [
                    S[  start_of_previou_line  + len("\ndef "):
                        end_of_previou_line - len(":")]
                    ]
        (a, b) = FindBlock(S, CommentStart, CommentEnd, b + len(CommentEnd))
    return Result
def GetCommentBlocks(S, CommentStart, CommentEnd):
    Result = []
    (a, b) = FindBlock(S, CommentStart, CommentEnd, 0)
    BlockIndex = 0
    while(a != -1):
        #print("-"*10 +  " Block "+ str(BlockIndex) +  "-"*20)
        Block = S[a + len(CommentStart) : b]
        Result += [Block]
        BlockIndex += 1
        (a, b) = FindBlock(S, CommentStart, CommentEnd, b + len(CommentEnd))
    return Result
def GetExampleBlocks(S, ExampleStart, ExampleEnd, trim):
    Result = []
    (a, b) = FindBlock(S, ExampleStart, ExampleEnd)
    BlockIndex = 0
    while(a != -1):
        #print("-"*10 +  " Block "+ str(BlockIndex) +  "-"*20)
        Block = S[a + len(ExampleStart) : b]
        if trim:
            Block = Block[trim:]
            Block = Block.replace("\n"+ " "*trim, "\n")
        Result += [Block]
        BlockIndex += 1
        (a, b) = FindBlock(S, ExampleStart, ExampleEnd, b + len(ExampleEnd))
    return Result

def go(TestFile, InitScript, trim = 0,
        CommentStart = "\"\"\"",
        CommentEnd = "\"\"\"",
        ExampleStart = "~>\n",
        ExampleEnd = "# End of Example\n",
        def_name = "eg"):
    Output = ""
    Output += InitScript
    f = open(TestFile, "r")
    Content = f.read()

    Functions = GetTheLineBeforeCommentBlocks(Content, CommentStart, CommentEnd)
    Comments = GetCommentBlocks(Content, CommentStart, CommentEnd)
    NumberOfExample = 0
    i = 0
    for comment in Comments:
        examples = GetExampleBlocks(comment, ExampleStart, ExampleEnd, trim)
        if len(examples):
            for eg in examples:
                Output +=   "\n"\
                            +"# Example "\
                            + str(NumberOfExample)\
                            + " in "\
                            + Functions[i]\
                            + "\n"\
                            + "def "\
                            + def_name\
                            + "_"\
                            + str(NumberOfExample)\
                            + "():\n"\
                            + eg
                NumberOfExample += 1
        i+=1
    #Write down all the functions
    Output += "\n"
    for egnum in range(NumberOfExample):
        Output += "eg_" + str(egnum) + "()\n"
    f.close()
    return Output

TestFile = "lukis.py"
OutputFile = "Testme.py"
InitScript = """
#!python3
# Codes generated by GenTestCode.py to test
# the example listesd in the Testfile.

from lukis import *
"""
g = open(OutputFile, "w")
g.write(
    go(TestFile, InitScript)
        )

g.close()
print("Finish. :)")



# The spelling of the command used in LUKIS stream

#The followings are just like atoms
# pdf generic commands
LineTo = "LineTo"
MoveTo = "MoveTo"
Stroke = "Stroke"
Close = "Close"
Close_Stroke = "Close_Stroke"
Fill = "Fill"
Fill_Stroke = "Fill_Stroke"
Close_Fill_Stroke = "Close_Fill_Stroke"
Scale = "Scale"
Translate = "Translate"
CurveTo = "CurveTo"
RotateCoordinate = "RotateCoordinate"
SetLineWidth = "SetLineWidth"
SetDash = "SetDash"
SetLineJoin = "SetLineJoin"
SetLineColor = "SetLineColor"
SetFillColor = "SetFillColor"
SetFont = "SetFont"
Text= "Text"
Picture = "Picture"



#!python3
"""
=====
Lukis
=====

Description:
~~~~~~~~~~~~
* Generate device independent vector graphic stream
  that can be converted to pdf, svg, or other vector graphic format.

* The output stream is in pure txt, and csv format.

* A simple stream looks like this::

    MoveTo,50,50
    LineTo,150,150
    Stroke

* "Lukis" is the Malay word for "Draw"

* show(GStream) converts the graphic stream to pdf, svg or other
  graphic file, and open it using the default viewer.

Example:
--------
::

    #To make a line from (0,0) to (300,300) and display the graphics:
    S = moveto(0,0)\\
    + lineto(300,300)\\
    + stroke()
    show(S)


Installation
~~~~~~~~~~~~
Python 3 as default Python::

	pip install lukis

Python as Python3::

	pip3 install kopi


Coding habits:
--------------
* For fixed/static length array inputs like (width, height), (origin_x, origin_y) etc,
  uses tuple (__, __)
* For dynamic length array like [x0, x1, x2, ...xn],
  use list or numpy array [__, __, ...]

"""
__all__ = ["lineto", "moveto", "curveto", "stroke", "show",
            "line","setlinewidth", "setlinecolor",
            "setfillcolor",
            "move", "rotate",
            "setfont", "text", "ftext",
            "rectangle"]


import os
import numpy as np
from . import command
from . import myfont
from . import makepdf
"""
    LUKIS stream : csv format
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    command,parameter,...parameter EOL
    :
    :
    command,parameter,...parameter EOL
"""
"""
    Color setting
    ~~~~~~~~~~~~~
    Follow the method used in pdf to set the
    -color of stroking- and the -color of nonstroking-
"""
#Global constants
SPACE = " "
TAB = " "*4
COMMA = ","
QUOTE = "\""
QUOTE_SINGLE = "\'"
EOL = "\n"

# Global parameters. Use them cautiously.
_Current_FillColor_ = (0, 0, 0)
_Current_LineColor_ = (0, 0, 0)
_Current_LineWidth_ = 1
_Current_FontSize_ = 12
_Current_Font_ = "Helvetica"
_DEFAULT_PAPER_ = (500,400) #(width, height)

class font:
    Times_Roman           = "Times-Roman"
    Times_Bold            = "Times-Bold"
    Times_Italic          = "Times-Italic"
    Times_BoldItalic      = "Times-BoldItalic"
    Helvetica             = "Helvetica"
    Helvetica_Bold        = "Helvetica-Bold"
    Helvetica_Oblique     = "Helvetica-Oblique"
    Helvetica_BoldOblique = "Helvetica-BoldOblique"
    Courier               = "Courier"
    Courier_Bold          = "Courier-Bold"
    Courier_Oblique       = "Courier-Oblique"
    Courier_BoldOblique   = "Courier-BoldOblique"
    Symbol                = "Symbol"
    Symbol_ZapfDingbats   = "ZapfDingbats"

def getcurrent_fillcolor():
    return _Current_FillColor_
def getcurrent_linecolor():
    return _Current_LineColor_

def form(*parameters):
    """
    string form( * string)

    write the arguments in the LUKIS stream format
    e.g.
        form("lineto", "1", "2") ==> "lineto,1,2\n"
    """
    S = ""
    for i in range(len(parameters)):
        S += parameters[i]
        if i < len(parameters) - 1:
            S += COMMA
    S+= EOL
    return S

def ftos (Float_Value):
    """
    string ftos(float)

    convert float to string
    Instead of using python's ftos(), use this one to control the precision
    my Float to String mini function

    Precision = %.4f
    Unnecessary 0 will be discarded
    """
    S= "%.4f" %Float_Value
    n = len(S)
    if S[n-5] == '.' :
        if S[ n-4 : n ] =="0000":
            S = S[ 0 : n-5 ]
        elif S[n-3:n] == "000":
            S = S[ 0 : n-3]
        elif S[n-2:n] == "00":
            S = S[ 0 : n-2]
        elif S[n-1] == "0":
            S = S[ 0 : n-1]
    return S

# ----------------------------------------------------
# Basic and generic building block
def lineto(x, y):
    """
    lineto(x,y) creates a new point at (x,y)
    and make a line path to that point.

    Example
    ~~~~~~>
    #draw a simple line
    S = moveto(50,50)
    S += lineto(150,150)
    S += stroke()
    show(S)
    # End of Example

    """
    return form(command.LineTo, ftos(x), ftos(y) )

def moveto(x, y):
    """
    moveto(x,y) moves the path to (x,y) without creating a line.
    """
    return form(command.MoveTo, ftos(x), ftos(y) )

def curveto(x1, y1, x2, y2, x3, y3):
    """
    curveto(x1, y1, x2, y2, x3, y3) create a three points Bezier curve.
    """
    return form(command.CurveTo,
                ftos(x1),
                ftos(y1),
                ftos(x2),
                ftos(y2),
                ftos(x3),
                ftos(y3)
                )

def stroke():
    """
    stroke() draws the line for the defined paths
    """
    return form(command.Stroke)

def close():
    return form(command.Close)

def close_stroke():
    return form(command.Close_Stroke)

def fill():
    return form(command.Fill)

def close_fill_stroke():
     return form(command.Close_Fill_Stroke)

def fill_stroke():
    return form(command.Fill_Stroke)

def translate_coordinate(x, y):
    """
    Translate the coordinate to x,y
    """
    return form(command.Translate, ftos(x), ftos(y))

def rotate_coordinate(angle):
    return form(command.RotateCoordinate,
                ftos(angle)
                )
def scale(S, scalex=1, scaley=1):
    if scalex == 1 and scaley == 1:
        return S
    return form(command.Scale, ftos(scalex), ftos(scaley))\
            + S\
            + form(command.Scale, "1", "1")

def setlinecolor(R, G, B):
    """
    R, G, B are decimals from 0 ~ 255
    """
    global _Current_LineColor_
    _Current_LineColor_ = (R,G,B)
    return form(command.SetLineColor,
                ftos(R),
                ftos(G),
                ftos(B))
def setfillcolor(R, G, B):
    """
    R, G, B are decimals from 0 ~ 255

    P.S. Fill color is also the font color for text
    """
    global _Current_FillColor_
    _Current_FillColor_ = (R,G,B)
    return form(command.SetFillColor,
                ftos(R),
                ftos(G),
                ftos(B))
def setlinewidth(linewidth):
    global _Current_LineWidth_
    _Current_LineWidth_ = linewidth
    return form(command.SetLineWidth, ftos(linewidth) )
def insertpicture(x,y,width, height, filename):
    """
    Insert the picture by filename
    x : x in pt
    y : y in pt
    width : the width of the picture
    height : the height of the picture
    """
    return form(command.Picture,
                ftos(x),
                ftos(y),
                ftos(width),
                ftos(height),
                QUOTE + filename + QUOTE)


# ----------------------------------------------------
# Non generic translation
def rotate(S, angle, dx = 0, dy = 0):
    """
    Rotates the object S for an angle (degree) about point (dx, dy)
    """
    if dx == 0 and dy == 0:
        return rotate_coordinate(angle)\
                + S\
                + rotate_coordinate(-angle)
    S = move(S, -dx, -dy)
    S = rotate(S, angle, 0, 0)
    S = move(S, dx, dy)
    return S

def move(S, x, y):
    """
    Move the object defined in S to x,y
    """
    return translate_coordinate(x,y)\
            + S\
            + translate_coordinate(-x, -y)



def line(x1, y1, x2, y2):
    return moveto(x1, y1)\
            + lineto(x2, y2)\
            + stroke()

def circle(x, y, radius, border = True, solid = True):
    magic = 0.551915024494
    S = ""
    S = moveto(    x, y + radius )

    S += curveto(
                x + magic*radius,
                y + radius,
                x + radius,
                y + magic*radius,
                x + radius,
                y + 0)

    S += curveto(
                x + radius,
                y - 1.0*magic*radius,
                x + magic*radius,
                y - 1.0*radius,
                x + 0,
                y - 1*radius)

    S += curveto(
                x - 1.0*magic*radius,
                y - 1*radius,
                x - 1.0*radius,
                y - 1*magic*radius,
                x - 1*radius,
                y + 0)

    S += curveto(
                x - 1*radius,
                y + magic*radius,
                x - 1*magic*radius,
                y + radius,
                x + 0,
                y + radius)

    if solid == True:
        if border == True:
            S += fill_stroke()
        else:
            S += fill()
    else:
        if border == True:
            S += stroke()
        else:
            pass
    return S

def rectangle(x1, y1, x2, y2, border = True, solid = True):
    S = moveto(x1, y1)\
        + lineto(x2, y1)\
        + lineto(x2, y2)\
        + lineto(x1, y2)\
        + close()
    if solid == True:
        if border == True:
            S += fill_stroke()
        else:
            S += fill()
    else:
        if border == True:
            S += stroke()
        else:
            pass
    return S

def setfont(font, fontsize):
    global _Current_Font_
    _Current_Font_ = font
    global _Current_FontSize_
    _Current_FontSize_ = fontsize
    return form(
            command.SetFont,
            font,
            ftos(fontsize)
            )
def text(x,y, thetext):
    """
    -------------------
    Put a text at (x,y)
    -------------------

    Example:
    ~~~~~~~>
    # Simple Text
    S = setfont("Helvetica", 16)
    S += text(100,200, "This is Helvetica font size = 16")
    S += setlinecolor(30,30,30)\
         + setlinewidth(0.5)
    S += line(100, 200, 400, 200)
    S += setfont("Times-Roman", 12)
    S += text(100, 100, "This is Time regular")
    S += line(100, 100, 400, 100)
    show(S)
    # End of Example
    """
    return form(command.Text,
            ftos(x),
            ftos(y),
            QUOTE + thetext + QUOTE,
            )
def ftext(x,y,thetext, alignment = 0, rotation = 0, font = None, fontsize = None):
    """
    ---------------------------------------------------
    Create formated text. Allow Rotation, and Alignment
    ---------------------------------------------------

    x, y     : The origin position
    alignment:
            0   -> left aligned
            0.5 -> center aligned
            1   -> right aligned
            p.s. Also allows non-zero values
    rotation : angle in degree
    font     : font to be set for the use only in this function,
               i.e. the current font will be set to this font,
                    and then reset back after use.
                Use the current font if None
    fontsize : fontsize to be set for the use only in this function,
               i.e. the current fontsize will be set to this font,
                    and then reset back after use.
                Use the current fontsize if None

    Example
    ~~~~~~~>
    # Rotation of text
    L = line(100, 200, 300, 200)
    S = setfont("Helvetica", 16)
    S += ftext(100,200, "Hello", 0, 45)
    S += setfont("Times-Roman", 14)
    S += text(150, 220, "45 degree")
    S += setlinecolor(200,0,0)\
            + setlinewidth(0.5)
    S += L
    S += rotate( L, 45, 100, 200)
    show(S)
    # End of Example

    Example:
    ~~~~~~~>
    # Alignment of text
    L =  setlinewidth(0.4)\
        + setlinecolor(0, 100, 0)\
        + line(100, 100, 300, 100)\
        + line(100, 200, 300, 200)\
        + line(100, 300, 300, 300)\
        + line(200, 100, 200, 300)
    T =   ftext(200,100,"Helvetica14 (align = 0)", 0, 0, "Helvetica", 14)\
        + ftext(200,200,"Times-Bold13 (align = 0.5)", 0.5, 0, "Times-Bold", 13)\
        + ftext(200,300,"Courier12 (align = 1.0)", 1, 0, "Courier", 12)

    Caption = setfillcolor(255, 0, 0)\
              + ftext(100,350, "Alignment: xo = xo - alignment*(text width)")
    show(L + T + Caption)
    # End of Example
    """
    S =""
    xo = 0
    yo = 0

    if font is not None:
        Backup_Font = _Current_Font_
        Backup_FontSize = _Current_FontSize_
        if fontsize is not None:
            S += setfont(font, fontsize)
        else:
            S += setfont(font, _Current_FontSize_)

    else:
        if fontsize is not None:
            Backup_Font = _Current_Font_
            Backup_FontSize = _Current_FontSize_
            S += setfont(_Current_Font_, fontsize)
        else:
            pass

    if alignment != 0:
        xo = xo - alignment * \
            myfont.GetStringWidth(thetext,
                                    _Current_Font_,
                                    _Current_FontSize_)
    S += text(xo,yo, thetext)
    if rotation != 0:
        S = rotate(S, rotation)
    S = move(S, x, y)

    #Clean up
    if font is None and fontsize is None:
        pass
    else:
        S += setfont(Backup_Font, Backup_FontSize)
    return S

def put(symbol, x,y,rotation=0, scalex = 1, scaley=1):
    """
        Put the symbol, or text, or graphics defined in valid LUKIS stream
        at x, y, with rotation and scale
    """
    return move(
                rotate(
                    scale(symbol, scalex, scaley),
                    rotation),
                x, y)


def show(LukisStream,
        papersize = _DEFAULT_PAPER_,
        show=True,
        format = "pdf",
        filename = "lukis.pdf",
        viewer = "open"):
    """
    Convert the lukis stream to the defined format and store it in the
    filename.
    Open the file using the defined viewer if show == True

    LukisStream : The string stream that contains valid graphic defination
    papersize : The size of the output graphics
                Allow two input formats:
                papersize = "A4"
                papersize = (400, 500)
                            (WIDTH, HEIGHT)
    """
    makepdf.makepdf(
        LukisStream,
        PaperSize=papersize,
        PDF_Filename = filename
        )
    if show:
        os.system(viewer + " " + filename)

    return



#!python3
from setuptools import setup

setup(
   name='lukis',
   version='0.18.5',
   description='Generate a device independent vector graphic stream that can be converted to pdf, svg, or other vector graphic format.',
   author='CHEN, Lee Chuin',
   author_email='leechuin@gmail.com',
   license='MIT',
   python_requires='>=3',
   long_description=open('README.rst').read(),
   classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Education',
        'Topic :: Scientific/Engineering :: Visualization',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
   keywords='lukis vector graphic pdf svg',
   packages=['lukis'],  #same as name
   install_requires=['numpy','pillow'], #external packages as dependencies
)



#!python3
"""
    Generate pdf file from graphic stream

    Require the information of fonts stored in myfont.py
"""
import numpy as np
from datetime import datetime
import os
import zlib as gzip
from . import myfont
from . import myimage
from . import command

#Global constants
SPACE = " "
TAB = " "*4
COMMA = ","
QUOTE = "\""
QUOTE_SINGLE = "\'"
EOL = "\n"

def itos (Int_Value):
        return "%d" %Int_Value
def GetPaperSize(PaperSize, Unit="pt", Portrait=True):
    #Allow two input formats:
    #    eg.    PaperSize = "A4"
    #    or    PaperSize = [400, 500]
    #    return
    CommonPaperSizeByName = {
        "Letter"        :"612 792",
        "Tabloid"        :"792 1224",
        "Ledger"        :"1224 792",
        "Legal"            :"612 1008",
        "Statement"        :"396 612",
        "Executive"        :"540 720",
        "A0"            :"2384 3371",
        "A1"            :"1685 2384",
        "A2"            :"1190 1684",
        "A3"            :"842 1190",
        "A4"            :"595 842",
        "A5"            :"420 595",
        "B4"            :"729 1032",
        "B5"            :"516 729",
        "Folio"            :"612 936",
        "Quarto"        :"610 780",
        "10x14"            :"720 1008"
        }
    if isinstance(PaperSize, list) or isinstance(PaperSize, tuple):
        if len(PaperSize) ==2:
            if Unit=="pt":
                return itos(int(PaperSize[0])) +\
                    " " +\
                    itos(int(PaperSize[1]))
            else:
                return ftos(ToPt(PaperSize[0],Unit)) +\
                    " " +\
                    ftos(ToPt(PaperSize[1],Unit))
        else:
            print("Invalid Manual Paper Size Parameters." +
                    " Assume Letter")
            return CommonPaperSizeByName["Letter"]

    else:
        if PaperSize in CommonPaperSizeByName:
            S = CommonPaperSizeByName[PaperSize]
            if Portrait == False:
                # assume landscape. Inverse the position for Paper Size
                S = S.split(" ")
                S = S[1] + " " + S[0]

            print(CommonPaperSizeByName[PaperSize])
            return S

        else:
            print("Name Paper of Paper Size Not Valid. Asssume Letter")
            return CommonPaperSizeByName["Letter"]
def Decimal_To_10_DigitsString(value):
    Decimal = int(value) # just in case if user put something else.
    S = "%d"%Decimal
    AdditionalZerosd = "0" * (10- len(S) )
    return AdditionalZerosd + S
def DecimalToAsciiHex(decimal):
    S = "%x"%decimal
    if len(S) ==1:
        S = "0" + S
    return S

def ftos (Float_Value):
    """
    string ftos(float)

    convert float to string
    Instead of using python's ftos(), use this one to control the precision
    my Float to String mini function

    Precision = %.4f
    Unnecessary 0 will be discarded
    """
    S= "%.4f" %Float_Value
    n = len(S)
    if S[n-5] == '.' :
        if S[ n-4 : n ] =="0000":
            S = S[ 0 : n-5 ]
        elif S[n-3:n] == "000":
            S = S[ 0 : n-3]
        elif S[n-2:n] == "00":
            S = S[ 0 : n-2]
        elif S[n-1] == "0":
            S = S[ 0 : n-1]
    return S
def qsplit(S,
        separator = COMMA,
        quote_Double = QUOTE,
        quote_Single = QUOTE_SINGLE):
    """
    splits S according to separator,
    but ignores those within the quote marks
    """
    Location = []
    i = 0
    while i < len(S):
        #loop out until the end of QUOTE
        if (S[i] == quote_Double) | (S[i] == quote_Single) :
            quote_this = S[i]
            #print("hit q")
            i += 1
            while S[i] != quote_this:
                i += 1

        if S[i] == separator:
            Location += [i]
        i += 1

    if len(Location) == 0:
        return [S]
    Result = [
                S[:Location[0] ]
                ]
    for n in range(len(Location)):
        if n != len(Location) -1:
            Result += [
                        S[Location[n]+1 : Location[n + 1]]
                        ]
        else:
            Result += [
                        S[Location[n]+1 : ]
                        ]

    return Result

def lineto(parameters):
    """
    string lineto (string x, string y)
    """
    [x,y] = parameters
    return x\
            + SPACE\
            + y\
            + SPACE\
            + "l"\
            + EOL
def moveto(parameters):
    """
    string moveto (string x, string y)
    """
    [x,y] = parameters
    return x\
            + SPACE\
            + y\
            + SPACE\
            + "m"\
            + EOL
def curveto(parameters):
    [x1, y1, x2, y2, x3, y3] = parameters
    return x1\
            + SPACE\
            + y1\
            + SPACE\
            + x2\
            + SPACE\
            + y2\
            + SPACE\
            + x3\
            + SPACE\
            + y3\
            + SPACE\
            + "c"\
            + EOL
def stroke():
    return "S" + EOL
def close():
    return "h" + EOL
def close_stroke():
    return "s" + EOL
def fill():
    return "f" + EOL
def fill_stroke():
    return "B" + EOL
def close_fill_stroke():
    return "b" + EOL

def translate_coordinate(args):
    """
    This is the generic one
    """
    [tx, ty] = args
    return "1 0 0 1 "\
            + tx\
            + SPACE\
            + ty\
            + " cm"\
            + EOL
def rotate_coordinate(args):
    [angle] = args
    radian = float(angle) * np.pi/180

    return ftos( np.cos(radian))\
            + SPACE\
            + ftos( np.sin(radian))\
            + SPACE\
            + ftos(-1*np.sin(radian) )\
            + SPACE\
            + ftos(np.cos(radian))\
            + SPACE\
            + "0"\
            + SPACE\
            + "0"\
            + SPACE\
            + "cm"\
            + EOL
def scale(args):
    [scalex, scaley] = args
    return  scalex\
            + SPACE\
            + "0 0"\
            + SPACE\
            + scaley\
            + SPACE\
            + "0 0"\
            + " cm"\
            + EOL

def ToPdfRGB(R, G, B):
    """
    Tuple ToPdfRGB(Tuple RGB)

    Convert the standard RGB (0 ~ 255)
    to pdf normalized RGB (0 ~ 1)

    The RGB is 3 elements Tuple, i.e, (R, G, B).
    Returns pdfRGB in tupl, , i.e, (pdfR, pdfG, pdfB)
    """
    return (1.0*R/255, 1.0*G/255, 1.0*B/255)

def setrgb_stroking(args):
    R = int(args[0])
    G = int(args[1])
    B = int(args[2])
    (pdf_R, pdf_G, pdf_B) = ToPdfRGB (R, G, B)
    return ftos(pdf_R)\
            + SPACE\
            + ftos(pdf_G)\
            + SPACE\
            + ftos(pdf_B)\
            + SPACE\
            + "RG"\
            + EOL
def setrgb_nonstroking(args):
    R = int(args[0])
    G = int(args[1])
    B = int(args[2])
    (pdf_R, pdf_G, pdf_B) = ToPdfRGB (R, G, B)
    return ftos(pdf_R)\
            + SPACE\
            + ftos(pdf_G)\
            + SPACE\
            + ftos(pdf_B)\
            + SPACE\
            + "rg"\
            + EOL
def setlinewidth(args):
    [linewidth] = args
    return linewidth + SPACE + "w" + EOL
def setdash(args):
    return

# pdf generic
def text_setposition(x ,y):
    return x\
            + SPACE\
            + y\
            + SPACE\
            + "Td"\
            + EOL

def text_show(thetext):
    return "("\
            + thetext\
            + ")"\
            + SPACE\
            + "Tj"\
            + EOL

# Only need to set once in the pdf document
def setfont(args):
    [font, fontsize] = args
    return "BT"\
            + EOL\
            + "/"\
            + myfont.GetShortFontName(font)\
            + SPACE\
            + fontsize\
            + SPACE\
            + "Tf"\
            + EOL\
            + "ET"\
            + EOL
#Non pdf Generic

def Remove_Quotes(thetext, quote_Double = QUOTE,
    quote_Single = QUOTE_SINGLE):
    if thetext[0] == quote_Double:
        if thetext[-1] == quote_Double:
            return thetext[1:-1]
        else:
            print("Error removing quotes ! Do nothing")
            return thetext
    elif thetext[0] == quote_Single:
        if thetext[-1] == quote_Single:
            return thetext[1:-1]
        else:
            print("Error removing quotes ! Do nothing")
            return thetext
    else:
        print("Error removing quotes ! Do nothing")
        return thetext
def InsertEscapeBeforeParenthesis(thetext):
    thetext = thetext.replace("(", "\(")
    thetext = thetext.replace(")", "\)")
    return thetext
def text(args):
    [x, y, thetext] = args
    thetext = Remove_Quotes(thetext)
    thetext = InsertEscapeBeforeParenthesis(thetext)
    return "BT"\
            + EOL\
            + text_setposition(x,y)\
            + text_show(thetext)\
            + "ET"\
            + EOL

_PictureNumber_ = 0
def increasePictureNumber():
    global _PictureNumber_
    _PictureNumber_ += 1
def resetPictureNumber():
    global _PictureNumber_
    _PictureNumber_ = 0

_PictureFilename_ = []
def appendPictureFilename(f):
    global _PictureFilename_
    _PictureFilename_ += [f]
def resetPictureFilename():
    global _PictureFilename_
    _PictureFilename_ = []


def insertpicture(args):#x, y, w, h):
    [x, y, w, h, picturefilename] =  args

    #Eg: "400 0 0 400 200 150 cm\n"
    S    = "q\n" +\
        w + " 0 0 " +\
        h + " " +\
        x + " " +\
        y + " cm\n" +\
        "/InsertPic" + str(_PictureNumber_) + " Do\n"+\
        "Q\n"
    increasePictureNumber()

    #Get rid of the QUOTES mark
    picturefilename = picturefilename[1:-1]
    appendPictureFilename(picturefilename)
    return S

def ExeCommand(args):
    Command = args[0]
    parameters = args[1:]
    if Command == command.LineTo:
        return lineto(parameters)

    elif Command == command.MoveTo:
        return moveto(parameters)

    elif Command == command.Stroke:
        return stroke()

    elif Command == command.Close:
        return close()

    elif Command == command.Close_Stroke:
        return close_stroke()

    elif Command == command.Close_Fill_Stroke:
        return close_fill_stroke()

    elif Command == command.Fill:
        return fill()

    elif Command == command.Fill_Stroke:
        return fill_stroke()

    elif Command == command.Translate:
        return translate_coordinate(parameters)

    elif Command == command.CurveTo:
        return curveto(parameters)

    elif Command == command.RotateCoordinate:
        return rotate_coordinate(parameters)

    elif Command == command.Scale:
        return scale(parameters)

    elif Command == command.SetLineColor:
        return setrgb_stroking(parameters)

    elif Command == command.SetFillColor:
        return setrgb_nonstroking(parameters)

    elif Command == command.SetLineWidth:
        return setlinewidth(parameters)

    elif Command == command.SetDash:
        return setdash(parameters)

    elif Command == command.Text:
        return text(parameters)

    elif Command == command.SetFont:
        return setfont(parameters)
    elif Command == command.Picture:
        return insertpicture(parameters)

    elif Command == "":
        pass
        return ""
    else:
        print("Unknown Command !")
        print(Command)
        return ""

def lks2gs(LukisStream):
    """
    Convert LUKIS format (lks) to pdf graphic stream
    """
    R =""
    S = LukisStream.split(EOL)
    for s in S:
        s = qsplit(s)
        R += ExeCommand(s)
    return R

#------------------------------------------------------
def obj_definefont(ObjectNumber,
                    GenerationNumber,
                    BaseFont,
                    DefineFontName):
    S = str(ObjectNumber) + " " + str(GenerationNumber) + " obj\n"
    S += ("<</Type /Font\n"+
        "/Subtype /Type1\n"+
        "/Name /" + DefineFontName + "\n"+
        "/BaseFont /"+ BaseFont +"\n"+
        #"/Encoding /MacRomanEncoding\n"+
        #"/Encoding /WinAnsiEncoding\n"+
        ">>\n"+
        "endobj\n"
    )
    return S.encode()
def obj_greeting():
    S =""
    S +=    "%PDF-1.4\n"+\
            "% Created Manually\n"+\
            "%%Creator: Lee Chuin CHEN\n" +\
            "%% [leechuin@yamanashi.ac.jp, leechuin@gmail.com]\n" +\
            "%%Title: pplot\n" +\
            "%%CreationDate: "+ str(datetime.now()) + "\n"
    return S.encode()
def obj_catalog (objnum_base, objnum_outlines, objnum_pages):
    S = str(objnum_base) + " 0 obj\n"+\
        "<</Type /Catalog\n" +\
        "/Outlines " + str(objnum_outlines) + " 0 R\n" +\
        "/Pages " + str(objnum_pages) +" 0 R\n" +\
        ">>\n" +\
        "endobj\n"
    return S.encode()
def obj_outline(objnum_outlines):
    S = str(objnum_outlines) + " 0 obj\n" +\
        "<</Type /Outlines\n" +\
        "/Count 0\n" +\
        ">>" +\
        "endobj\n"
    return S.encode()
def obj_pages(objnum_pages, objnum_kids):
    #{{{
    S = str(objnum_pages) + " 0 obj\n" +\
        "<</Type /Pages\n" +\
        "/Kids [" + str(objnum_kids) + " 0 R]\n" +\
        "/Count 1\n" +\
        ">>\n" +\
        "endobj\n"
    return S.encode()
def obj_pagetype(
        objnum_kids,
        objnum_pages,
        objnum_fontbase,
        objnum_contents,
        objnum_picture=0,
        objnum_dataimage=0,
        PaperSize="Letter",
        Unit="pt",
        ):
    #DeclareFont = [    ["FH"    ,    fontname.Helvetica],
    #                    ["FHB"    ,    fontname.Helvetica_Bold],
    #                    ...
    DeclaredFont = myfont.ShortFontName.items()
    S_AllFontnames= ""
    S_Xobject = ""


    FontPointer = objnum_fontbase

    GotDataImage = True if objnum_dataimage!=0 else 0
    if len(_PictureFilename_)  | GotDataImage :
        S_Xobject += "/XObject <<\n"
        if len(_PictureFilename_):
            for n in range(_PictureNumber_):
                S_Xobject +=\
                "/InsertPic" + itos(n) + " " +\
                str(objnum_picture + n ) +\
                " 0 R\n"
        if GotDataImage:
            S_Xobject +=\
                "/DataImg0" + " " +\
                str(objnum_dataimage) +\
                " 0 R\n"
        S_Xobject += ">>\n"
    #e.g : /FT 40 0 R
    for f in DeclaredFont:
        S_AllFontnames += "/" + f[1] + " " + str(FontPointer) + " 0 R\n"
        FontPointer = FontPointer + 1
    S = (str(objnum_kids) + " 0 obj\n" +\
        "<</Type /Page\n" +\
        "/Parent " + str(objnum_pages) + " 0 R\n" +\
        #The code to adjust the paper size
        #"/MediaBox [0 0 612 792]\n" +
        "/MediaBox [0 0 " +\
        GetPaperSize(PaperSize,Unit) + "]\n"+\
        "/Contents " + str(objnum_contents) + " 0 R\n" +\
        "/Resources<</ProcSet [/PDF /Text]\n"+\
        "/Font <<\n"+\
            S_AllFontnames+\
        ">>\n"+\
        S_Xobject +\

        ">>\n"+\
        ">>\n"+\
        "endobj\n")
    return S.encode()
def obj_endstream_endobj():
    return     ("\n"+
            "endstream\n"+
            "endobj\n").encode()
def obj_contents(objnum_contents, Stream, Compress):
    #The Stream here is Byte
    StreamLength = len(Stream)
    S = str(objnum_contents) + " 0 obj\n" +\
            "<< /Length %d "%StreamLength +"\n"+\
            ("/Filter /FlateDecode" if Compress else "") +\
            ">>\n"+\
            "stream\n"
    return S.encode()+\
             Stream +\
             obj_endstream_endobj()
def obj_image(objnum_picture, ImageWidth,
        ImageHeight, ImageStream, DCTDecode):
    ImageLength = len(ImageStream)
    S=    str(objnum_picture) + " 0 obj\n" +\
        "<< /Type /XObject\n"+\
        "/Subtype /Image\n"+\
        "/Width " + str(ImageWidth) + "\n" +\
        "/Height " + str(ImageHeight) + "\n" +\
        "/ColorSpace /DeviceRGB\n"+\
        "/BitsPerComponent 8\n"+\
        "/Length "+ str(ImageLength) + "\n"+\
        "/Filter ["+\
                "/FlateDecode "+\
                "/ASCIIHexDecode "+\
                ("/DCTDecode" if DCTDecode else "") +\
                "] >>\n"+\
        "stream\n"
    return S.encode()+\
            ImageStream+\
            obj_endstream_endobj()


def obj_xref(pdf_objects):
    #{{{
    # At last, about time to write the XREF
    # If it is not done correctly, some pdf reader
    #eg. GS will complaint.:(
    NumOfPdfObj = len(pdf_objects)
    offset = [0]*NumOfPdfObj
    for i in range(1, NumOfPdfObj):
        offset[i] = offset[i-1] + len(pdf_objects[i-1])

    startxref = offset[NumOfPdfObj-1] + len(pdf_objects[NumOfPdfObj -1])

    Xref=""
    for i in range(1 , NumOfPdfObj):
        Xref += Decimal_To_10_DigitsString(offset[i]) + " 00000 n\r\n"

    pdf_xref = ("xref\r\n"+
        "0 "+ str(NumOfPdfObj) +" \r\n"+
        "0000000000 65535 f\r\n"+
        Xref +
        "trailer\n"+
        "<</Size 8\n"+
        "/Root 1 0 R\n"+
        ">>\n"+
        "startxref\n"+
        "%d"%(startxref) + "\n"
        "%%%EOF\n")
    return pdf_xref.encode()

def makepdf(GraphicsStream,
        Compress=False,
        UseJPEGForJPEGFile = False,
        #ImageData=[[128,128,128],[128,128,128],[34,70,24],[67,34,90]]):
        ImageData=[],
        ImageDataWidth =0,
        ImageDataHeigth =0,
        PaperSize="Letter",
        Unit="pt",
        PaperColor="white",
        PDF_Filename="killmex.pdf"):
    """
    Convert LUKIS graphic stream to pdf
    """
    resetPictureNumber()
    resetPictureFilename()
    GraphicsStream = lks2gs(GraphicsStream)
    GraphicsStream=GraphicsStream.encode()
    GotImageData = True if len(ImageData)>0 else False
    NumberOfImageData = 1 if GotImageData else 0
    if Compress:
        # in python 3, Zlib takes only Byte data. No more string.
        GraphicsStream = gzip.compress(GraphicsStream)

    # Manually assignig pdf obj number
    objnum_base     = 1
    objnum_outlines    = objnum_base + 1
    objnum_pages    = objnum_outlines + 1
    objnum_kids        = objnum_pages + 1
    objnum_contents = objnum_kids + 1


    objnum_picture = 0
    objnum_dataimage = 0
    if len(_PictureFilename_):
        objnum_picture    = objnum_contents + 1
        if GotImageData:
            objnum_dataimage =\
                 objnum_picture + NumberOfPicture
            objnum_fontbase = objnum_dataimage + 1
        else:
            objnum_fontbase =\
                 objnum_picture + _PictureNumber_
    else:
        if GotImageData:
            objnum_dataimage= objnum_contents + 1
            objnum_fontbase = objnum_dataimage + 1
        else:
            objnum_fontbase = objnum_contents + 1

    # Start writing pdf objects
    NumOfPdfObj = 1
    pdf_objects = []
    pdf_objects.append(obj_greeting())

    pdf_objects.append(obj_catalog (
                        objnum_base,
                        objnum_outlines,
                        objnum_pages
                        )
                    )

    pdf_objects .append(
         obj_outline(objnum_outlines)
        )

    pdf_objects.append(
        obj_pages(    objnum_pages,
                    objnum_kids)
            )

    pdf_objects.append(
        obj_pagetype(
            objnum_kids,
            objnum_pages,
            objnum_fontbase,
            objnum_contents,
            objnum_picture if len(_PictureFilename_) else 0,
            objnum_dataimage,
            PaperSize,
            Unit,
            )
        )



    pdf_objects.append(obj_contents(objnum_contents,
                        GraphicsStream,
                        Compress ))

    #Start to  insert picture from file
    if len(_PictureFilename_):
        for n in range(_PictureNumber_):
            [imagewidth,
            imageheight,
            imagelength,
            imagestream] = [0,0,0,"".encode()]
            ImageFilename = _PictureFilename_[n]
            DCTDecode =\
                 True if myimage.IsJPGFile(ImageFilename) &\
                UseJPEGForJPEGFile else False

            RetrievedImageData =\
                     myimage.GetImageByteStreamFromFile(
                    ImageFilename,
                    DCTDecode
                    )
            if RetrievedImageData == -1:
                print(
                    "Can not get image stream for \"" +
                    ImageFilename + "\"")
            else:
                [imagestream,
                imagewidth,
                imageheight]  =    RetrievedImageData

            pdf_objects.append(
                obj_image(
                        (objnum_picture + n),
                        imagewidth,
                        imageheight,
                        imagestream,
                        DCTDecode
                        )
                )

    if GotImageData:
        imagestream = myimage.GetImageByteStreamFromData(
            ImageData)
        pdf_objects.append(
                obj_image(
                        (objnum_dataimage),
                        ImageDataWidth,
                        ImageDataHeigth,
                        imagestream,
                        False
                        )
                )

    DeclareFont = myfont.ShortFontName.items()
    # Insert all available fonts
    FontPointer = objnum_fontbase
    for f in DeclareFont:
        pdf_objects.append(
                obj_definefont( FontPointer,
                                0,
                                f[0],    # Full name
                                f[1]    # Short name
                                )
                )
        FontPointer = FontPointer +  1

    #Finally, time to write the xref
    pdf_xref = obj_xref(pdf_objects)

    with open(PDF_Filename, "wb") as TargetFile:
        for myobject in pdf_objects:
            TargetFile.write(myobject)
        TargetFile.write(pdf_xref)
    return



#! python3
# To save my life time, I use the existing Image Processing Library
# PIL.
# Maintanence of orginal PIL is discontinued,
# so it could no longer be installed with pip.
# Use  PIL fork : Pillow instead
# e.g.:
# sudo easy_install pip
# sudo pip install pillow

__all__ = ["GetImageByteStreamFromData",
            "GetImageByteStreamFromFile",
            "IsJPGFile"]

from PIL import Image
#from .misc import *
import zlib as gzip
import binascii

def DecimalToAsciiHex(decimal):
    S = "%x"%decimal
    if len(S) ==1:
        S = "0" + S
    return S
def IsJPGFile(Filename):
    if Filename[Filename.rfind('.'):len(Filename)] == ".jpg":
        return True
    else:
        return False

def GetImageByteStreamFromData(ImageData):
    #Data format: array of [[R,G,B], ...]
    # E.g.
    #ImageData=[[128,128,128],[128,128,128],[34,70,24],[67,34,90]]):
    imgS = ""
    for n in range(len(ImageData)):
        for m in range(3):
            imgS += DecimalToAsciiHex(round (ImageData[n][m]))
    imgS += ">" #End of Data
    imgS = imgS.encode()
    imgS = gzip.compress(imgS)
    return imgS

# use PIL to peek on the file
def GetImageByteStreamFromFile(ImageFilename, DCTDecode=False):
    try:
        fa = Image.open(ImageFilename)
        ImageWidth = fa.size[0]
        ImageHeight = fa.size[1]
    except:
        print("Error Opening Image file !")
        return -1
    if DCTDecode & IsJPGFile(ImageFilename):
    #'Decompresses data encoded using a DCT
    #(discrete cosine transform) technique based on the JPEG standard,
    #reproducing image sample data that approximates the original data.'
    #from pdf reference 1.7
    # That mean I will just leave the jpg file as it is.
    # ANyway, need to hexilify the data.
        try:
            with open(ImageFilename, "rb") as fb:
                imgS = binascii.hexlify( fb.read())
                imgS += ">".encode()
        except:
            PrintError("Error Opening/Reading JPG file !")
            return -1
    else:
        imgD = list(fa.getdata())
        imgS = ""
        for n in range(len(imgD)):
            for m in range(3):
                imgS += DecimalToAsciiHex(imgD[n][m])
        imgS += ">" #End of Data
        imgS = imgS.encode()
    #Finish
    imgS = gzip.compress(imgS)
    return [imgS, ImageWidth, ImageHeight]



#!python3
"""
    ====
    Plot
    ====
    A standalone light module to plot simple 2D gragh.
    It relied on using lukis.py.


    Example :
    ~~~~~~~~
    # plot simple line

    ~ End of example ~
"""
import numpy as np
from . import lukis as lk
from . import mypretty
from . import myfont
__all__ = ["plot", "example1"]

_PlotSize_ = (350, 260)
_Origin_ = [80,80]
_DEFAULT_PAPER_ = (500,400)
# Commands that are reused from lukis.py
def setlinewidth(linewidth):
    return lk.setlinewidth(linewidth)
def setlinecolor(R, G, B):
    return lk.setlinecolor(R, G, B)
def setfillcolor(R, G, B):
    return lk.setfillcolor(R, G, B)
def show(LukisStream,
        papersize = _DEFAULT_PAPER_,
        show=True,
        format = "pdf",
        filename = "lukis.pdf",
        viewer = "open"):
    return lk.show(LukisStream,
            papersize ,
            show,
            format,
            filename,
            viewer)
def pretty(data_array):
    return mypretty.pretty(data_array)

def PosValueOnly(value):
    return value if value > 0 else 0


def map2pt(array, tickmin, tickmax, origin, length_pt):
    v_pt_o = origin
    v_pt = v_pt_o + (array - tickmin)/(tickmax - tickmin)*length_pt
    return v_pt
def mapXY2pt(xarray, yarray,
        xmin, xmax, ymin, ymax,
        plotsize = _PlotSize_,
        origin = _Origin_):
    # just in case
    xarray = np.array(xarray)
    yarray = np.array(yarray)
    (plotwidth, plotheight) = plotsize
    (origin_x, origin_y) = origin

    x_pt = map2pt(xarray, xmin, xmax, origin_x, plotwidth)
    y_pt = map2pt(yarray, ymin, ymax, origin_y, plotheight)
    return (x_pt, y_pt)
def plot(xarray, yarray,limit,plotsize = _PlotSize_,origin = _Origin_):
    (xmin, xmax, ymin, ymax) = limit
    (x_pt, y_pt) = mapXY2pt(xarray, yarray,
                            xmin, xmax,
                            ymin, ymax,
                            plotsize,
                            origin)
    S = ""
    S += lk.moveto(x_pt[0], y_pt[0])
    for i in range(len(xarray)-1):
        if xarray[i] >= xmin and xarray[i] <= xmax:
            S += lk.lineto(x_pt[i+1], y_pt[i+1])
    S += lk.stroke()
    return S
def point(xarray, yarray, size, limit, plotsize = _PlotSize_,origin = _Origin_):
    (xmin, xmax, ymin, ymax) = limit
    (x_pt, y_pt) = mapXY2pt(xarray, yarray,
                            xmin, xmax,
                            ymin, ymax,
                            plotsize,
                            origin)
    S = ""
    for i in range(len(x_pt)):
        x = x_pt[i]
        y = y_pt[i]
        S += lk.circle(x, y, size, border = True, solid = True)
    return S
def axis(axisnumber, tickarray,
            label = "",
            ticklength = 6,
            plotsize = _PlotSize_,
            origin = _Origin_,
            font = "Helvetica",
            fontsize = 8,
            labelfont = "Helvetica",
            labelfontsize = 12,
            offset = 0):
    (plotwidth, plotheight)  = plotsize
    (origin_x, origin_y) = origin

    S = ""
    if axisnumber == 0:
        # lower X axis
        S += lk.line(origin_x,
                    origin_y - offset,
                    origin_x + plotwidth,
                    origin_y - offset
                    )
    elif axisnumber == 1 :
        # left Y axis
        S += lk.line(origin_x  - offset,
                    origin_y,
                    origin_x - offset,
                    origin_y + plotheight
                    )
    elif axisnumber == 2 :
        # upper X axis
        S += lk.line(origin_x,
                    origin_y + plotheight + offset,
                    origin_x + plotwidth,
                    origin_y + plotheight + offset
                    )
    elif axisnumber == 3 :
        # right Y axis
        S += lk.line(origin_x + plotwidth + offset,
                    origin_y,
                    origin_x + plotwidth + offset,
                    origin_y + plotheight
                    )
    #Draw tick
    if axisnumber == 0 or axisnumber == 2 :
        XorY = 0
    else:
        XorY = 1

    ta_pt = map2pt(tickarray,
                    tickmin = tickarray[0],
                    tickmax = tickarray[-1],
                    origin = origin[XorY],
                    length_pt = plotsize[XorY])

    S += lk.setfont(font, fontsize)
    if axisnumber == 0:
        for i in range(len(ta_pt)):

            S += lk.line(ta_pt[i],
                        origin_y - offset,
                        ta_pt[i],
                        origin_y - ticklength - offset)
            S += lk.ftext(ta_pt[i],
                        origin_y - PosValueOnly(ticklength) - fontsize - offset,
                        thetext = str(tickarray[i]),
                        alignment = 0.5
                        )
        # The label
        S += lk.ftext(  origin_x + plotwidth/2,
                        origin_y  -  PosValueOnly(ticklength) - offset - fontsize - labelfontsize,
                        thetext = label,
                        alignment = 0.5,
                        font = labelfont,
                        fontsize = labelfontsize)
    elif axisnumber == 1:
        LongestTickValue = 0
        for i in range(len(ta_pt)):
            TickText = str(tickarray[i])
            S += lk.line(origin_x - offset,
                        ta_pt[i],
                        origin_x - ticklength -offset,
                        ta_pt[i])
            S += lk.ftext(origin_x - PosValueOnly(ticklength) - 0.3*fontsize -offset,
                        ta_pt[i] - fontsize*0.25,
                        thetext = TickText,
                        alignment = 1)
            ThisTickTextWidth = myfont.GetStringWidth(TickText, labelfont, labelfontsize)
            if ThisTickTextWidth > LongestTickValue:
                LongestTickValue = ThisTickTextWidth

        # The label
        S += lk.ftext(  origin_x -  PosValueOnly(ticklength) - offset - LongestTickValue,
                        origin_y  +  plotheight/2,
                        thetext = label,
                        alignment = 0.5,
                        rotation = 90,
                        font = labelfont,
                        fontsize = labelfontsize)
    elif axisnumber == 2:
        for i in range(len(ta_pt)):
            S += lk.line(ta_pt[i],
                        origin_y + plotheight + offset,
                        ta_pt[i],
                        origin_y + ticklength + plotheight + offset)
            S += lk.ftext(ta_pt[i],
                        origin_y + plotheight + PosValueOnly(ticklength) + 0.3*fontsize + offset,
                        thetext = str(tickarray[i]),
                        alignment = 0.5
                        )
            # The label
            S += lk.ftext(  origin_x + plotwidth/2,
                            origin_y  + PosValueOnly(ticklength) + offset + fontsize,
                            thetext = label,
                            alignment = 0.5,
                            font = labelfont,
                            fontsize = labelfontsize)
    elif axisnumber == 3:
        for i in range(len(ta_pt)):
            S += lk.line(origin_x + plotwidth + offset,
                        ta_pt[i],
                        origin_x + ticklength + plotwidth + offset,
                        ta_pt[i])
            S += lk.ftext(origin_x + plotwidth + ticklength + 0.3*fontsize + offset,
                        ta_pt[i] - fontsize*0.25,
                        thetext = str(tickarray[i]),
                        alignment = 0
                        )

    return S
def plotbox(plotsize = _PlotSize_, origin = _Origin_, color = (220,220,220) ):
    PreviousFillColor = lk.getcurrent_fillcolor()
    PreviousLineColor = lk.getcurrent_linecolor()
    S = ""
    S += lk.setfillcolor(color[0], color[1], color[2])
    x1 = origin[0]
    y1 = origin[1]
    x2 = x1 + plotsize[0]
    y2 = y1 + plotsize[1]
    S += lk.rectangle(x1, y1, x2, y2, border = False, solid = True)
    return S
def grid(axisnumber, tickarray, plotsize = _PlotSize_,origin = _Origin_):
    (origin_x,origin_y) = origin
    if axisnumber == 0:
        XorY = 0
    else:
        XorY = 1

    ta_pt = map2pt(tickarray,
                    tickmin = tickarray[0],
                    tickmax = tickarray[-1],
                    origin = origin[XorY],
                    length_pt = plotsize[XorY])
    S =""
    for i in range(len(ta_pt)):
        if axisnumber == 0:
            PlotHeight = plotsize[1]
            S += lk.line(ta_pt[i],
                        origin_y,
                        ta_pt[i],
                        origin_y + PlotHeight)

        elif axisnumber == 1:
            PlotWidth = plotsize[0]
            S += lk.line(origin_x,
                        ta_pt[i],
                        origin_x + PlotWidth,
                        ta_pt[i])
    return S

def example1():
    X1 = np.linspace(0, 5, 10)
    Y0 = X1
    Y1 = X1**2
    Y2 = X1**3

    # Set the plot limit using pretty
    # They can also be set manuallu too.
    xtick= pretty(X1)
    ytick= pretty(Y2)
    (xmin, xmax) = (xtick[0], xtick[-1])
    (ymin, ymax) = (ytick[0], ytick[-1])
    limit = (xmin, xmax, ymin, ymax)

    S = ""

    # Draw the plot background
    S += plotbox()
    S += setlinewidth(0.5)
    S += setlinecolor(255,255,255)
    S += grid(0, xtick)
    S += grid(1, ytick)


    #Plot the first curve, and put the data points
    S += setlinecolor(0,150,0)
    S += setfillcolor(0,260,0)
    S += plot(X1, Y0, limit)
    S += point(X1, Y0, size = 3, limit = limit)

    #On the same plot, draw the second curve
    S += setlinecolor(150,0,0)
    S += setfillcolor(220,0,0)
    S += plot(X1, Y1, limit)
    S += point(X1, Y1, size = 3, limit = limit)

    #On the same plot, draw the second curve
    S += setlinecolor(0,0,150)
    S += setfillcolor(0,0,260)
    S += plot(X1, Y2, limit)
    S += point(X1, Y2, size = 3, limit = limit)

    #Draw the axies
    S += setlinecolor(0,0,0)
    S += setfillcolor(0,0,0)
    S += axis(0, xtick, label = "xlabel", offset = 10)
    S += axis(1, ytick, label = "ylabel", offset = 10)
    S += axis(2, xtick, offset = 5)
    S += axis(3, ytick, offset = 5)
    show(S)
